@@ -44,7 +44,7 @@
 #include "drmtest.h"
 #include "intel_chipset.h"
 #include "intel_bufmgr.h"
-#include "intel_gpu_tools.h"
+#include "intel_io.h"
 #include "intel_batchbuffer.h"
 #include "intel_debug.h"
 #include "debug.h"
