@@ -47,10 +47,7 @@
 #include "drmtest.h"
 #include "igt_kms.h"
 
-#if defined(DRM_IOCTL_MODE_ADDFB2) && defined(DRM_I915_SET_SPRITE_COLORKEY)
-#define TEST_PLANES 1
-#include "drm_fourcc.h"
-#endif
+#include "ioctl_wrappers.h"
 
 /*
  * Mode setting with the kernel interfaces is a bit of a chore.
