@@ -39,7 +39,6 @@
 #include <sys/mman.h>
 #include <signal.h>
 #include <pciaccess.h>
-#include <getopt.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include <sys/wait.h>
@@ -51,20 +50,50 @@
 #include "drmtest.h"
 #include "i915_drm.h"
 #include "intel_chipset.h"
-#include "intel_gpu_tools.h"
+#include "intel_io.h"
 #include "igt_debugfs.h"
-#include "../version.h"
+#include "version.h"
 #include "config.h"
+#include "intel_reg.h"
+#include "ioctl_wrappers.h"
 
-/* This file contains a bunch of wrapper functions to directly use gem ioctls.
- * Mostly useful to write kernel tests. */
+/**
+ * SECTION:drmtest
+ * @short_description: Base library for drm tests and tools
+ * @title: drmtest
+ * @include: drmtest.h
+ *
+ * This library contains the basic support for writing tests, with the most
+ * important part being the helper function to open drm device nodes.
+ *
+ * But there's also a bit of other assorted stuff here.
+ *
+ * Note that this library's header pulls in the [i-g-t core](intel-gpu-tools-i-g-t-core.html)
+ * and [batchbuffer](intel-gpu-tools-intel-batchbuffer.html) libraries as dependencies.
+ */
+
+static int is_i915_device(int fd)
+{
+	drm_version_t version;
+	char name[5] = "";
+
+	memset(&version, 0, sizeof(version));
+	version.name_len = 4;
+	version.name = name;
+
+	if (drmIoctl(fd, DRM_IOCTL_VERSION, &version))
+		return 0;
+
+	return strcmp("i915", name) == 0;
+}
 
 static int
 is_intel(int fd)
 {
 	struct drm_i915_getparam gp;
-	int devid;
+	int devid = 0;
 
+	memset(&gp, 0, sizeof(gp));
 	gp.param = I915_PARAM_CHIPSET_ID;
 	gp.value = &devid;
 
@@ -74,8 +103,31 @@ is_intel(int fd)
 	return IS_INTEL(devid);
 }
 
+static void check_stop_rings(void)
+{
+	enum stop_ring_flags flags;
+	flags = igt_get_stop_rings();
+	igt_warn_on_f(flags != 0,
+		      "i915_ring_stop flags on exit 0x%x, can't quiescent gpu cleanly\n",
+		      flags);
+
+	if (flags)
+		igt_set_stop_rings(STOP_RING_NONE);
+}
+
 #define LOCAL_I915_EXEC_VEBOX	(4 << 0)
-/* Ensure the gpu is idle by launching a nop execbuf and stalling for it. */
+/**
+ * gem_quiescent_gpu:
+ * @fd: open i915 drm file descriptor
+ *
+ * Ensure the gpu is idle by launching a nop execbuf and stalling for it. This
+ * is automatically run when opening a drm device node and is also installed as
+ * an exit handler to have the best assurance that the test is run in a pristine
+ * and controlled environment.
+ *
+ * This function simply allows tests to make additional calls in-between, if so
+ * desired.
+ */
 void gem_quiescent_gpu(int fd)
 {
 	uint32_t batch[2] = {MI_BATCH_BUFFER_END, 0};
@@ -83,6 +135,8 @@ void gem_quiescent_gpu(int fd)
 	struct drm_i915_gem_execbuffer2 execbuf;
 	struct drm_i915_gem_exec_object2 gem_exec[1];
 
+	check_stop_rings();
+
 	handle = gem_create(fd, 4096);
 	gem_write(fd, handle, 0, batch, sizeof(batch));
 
@@ -132,9 +186,11 @@ void gem_quiescent_gpu(int fd)
 /**
  * drm_get_card:
  *
- * Get an intel card number for use in /dev or /sys
+ * Get an i915 drm card index number for use in /dev or /sys. The minor index of
+ * the legacy node is returned, not of the control or render node.
  *
- * Returns: -1 on error
+ * Returns:
+ * The i915 drm index or -1 on error
  */
 int drm_get_card(void)
 {
@@ -153,7 +209,7 @@ int drm_get_card(void)
 		if (fd == -1)
 			continue;
 
-		if (!is_intel(fd)) {
+		if (!is_i915_device(fd) || !is_intel(fd)) {
 			close(fd);
 			continue;
 		}
@@ -170,22 +226,23 @@ int drm_get_card(void)
 /** Open the first DRM device we can find, searching up to 16 device nodes */
 static int __drm_open_any(void)
 {
-	char *name;
-	int ret, fd;
+	for (int i = 0; i < 16; i++) {
+		char name[80];
+		int fd;
 
-	ret = asprintf(&name, "/dev/dri/card%d", drm_get_card());
-	if (ret == -1)
-		return -1;
+		sprintf(name, "/dev/dri/card%u", i);
+		fd = open(name, O_RDWR);
+		if (fd == -1)
+			continue;
 
-	fd = open(name, O_RDWR);
-	free(name);
+		if (is_i915_device(fd) && is_intel(fd))
+			return fd;
 
-	if (!is_intel(fd)) {
 		close(fd);
-		fd = -1;
 	}
 
-	return fd;
+	igt_skip("No intel gpu found\n");
+	return -1;
 }
 
 static int __drm_open_any_render(void)
@@ -205,7 +262,7 @@ static int __drm_open_any_render(void)
 		if (fd == -1)
 			continue;
 
-		if (!is_intel(fd)) {
+		if (!is_i915_device(fd) || !is_intel(fd)) {
 			close(fd);
 			fd = -1;
 			continue;
@@ -225,6 +282,7 @@ static void quiescent_gpu_at_exit(int sig)
 	if (at_exit_drm_fd < 0)
 		return;
 
+	check_stop_rings();
 	gem_quiescent_gpu(at_exit_drm_fd);
 	close(at_exit_drm_fd);
 	at_exit_drm_fd = -1;
@@ -235,11 +293,20 @@ static void quiescent_gpu_at_exit_render(int sig)
 	if (at_exit_drm_render_fd < 0)
 		return;
 
+	check_stop_rings();
 	gem_quiescent_gpu(at_exit_drm_render_fd);
 	close(at_exit_drm_render_fd);
 	at_exit_drm_render_fd = -1;
 }
 
+/**
+ * drm_open_any:
+ *
+ * Open an i915 drm legacy device node.
+ *
+ * Returns:
+ * The i915 drm file descriptor or -1 on error
+ */
 int drm_open_any(void)
 {
 	static int open_count;
@@ -251,12 +318,20 @@ int drm_open_any(void)
 		return fd;
 
 	gem_quiescent_gpu(fd);
-	at_exit_drm_fd = dup(fd);
+	at_exit_drm_fd = __drm_open_any();
 	igt_install_exit_handler(quiescent_gpu_at_exit);
 
 	return fd;
 }
 
+/**
+ * drm_open_any:
+ *
+ * Open an i915 drm render device node.
+ *
+ * Returns:
+ * The i915 drm file descriptor or -1 on error
+ */
 int drm_open_any_render(void)
 {
 	static int open_count;
@@ -269,197 +344,9 @@ int drm_open_any_render(void)
 	if (__sync_fetch_and_add(&open_count, 1))
 		return fd;
 
-	at_exit_drm_render_fd = dup(fd);
+	at_exit_drm_render_fd = __drm_open_any();
 	gem_quiescent_gpu(fd);
 	igt_install_exit_handler(quiescent_gpu_at_exit_render);
 
 	return fd;
 }
-
-/* signal interrupt helpers */
-static struct igt_helper_process signal_helper;
-long long int sig_stat;
-static void __attribute__((noreturn)) signal_helper_process(pid_t pid)
-{
-	/* Interrupt the parent process at 500Hz, just to be annoying */
-	while (1) {
-		usleep(1000 * 1000 / 500);
-		if (kill(pid, SIGUSR1)) /* Parent has died, so must we. */
-			exit(0);
-	}
-}
-
-static void sig_handler(int i)
-{
-	sig_stat++;
-}
-
-void igt_fork_signal_helper(void)
-{
-	if (igt_only_list_subtests())
-		return;
-
-	signal(SIGUSR1, sig_handler);
-
-	igt_fork_helper(&signal_helper) {
-		signal_helper_process(getppid());
-	}
-}
-
-void igt_stop_signal_helper(void)
-{
-	if (igt_only_list_subtests())
-		return;
-
-	igt_stop_helper(&signal_helper);
-
-	sig_stat = 0;
-}
-
-bool igt_env_set(const char *env_var, bool default_value)
-{
-	char *val;
-
-	val = getenv(env_var);
-	if (!val)
-		return default_value;
-
-	return atoi(val) != 0;
-}
-
-bool drmtest_dump_aub(void)
-{
-	static int dump_aub = -1;
-
-	if (dump_aub == -1)
-		dump_aub = igt_env_set("IGT_DUMP_AUB", false);
-
-	return dump_aub;
-}
-
-/* other helpers */
-void igt_exchange_int(void *array, unsigned i, unsigned j)
-{
-	int *int_arr, tmp;
-	int_arr = array;
-
-	tmp = int_arr[i];
-	int_arr[i] = int_arr[j];
-	int_arr[j] = tmp;
-}
-
-void igt_permute_array(void *array, unsigned size,
-			   void (*exchange_func)(void *array,
-						 unsigned i,
-						 unsigned j))
-{
-	int i;
-
-	for (i = size - 1; i > 1; i--) {
-		/* yes, not perfectly uniform, who cares */
-		long l = random() % (i +1);
-		if (i != l)
-			exchange_func(array, i, l);
-	}
-}
-
-void igt_progress(const char *header, uint64_t i, uint64_t total)
-{
-	int divider = 200;
-
-	if (!isatty(fileno(stderr)))
-		return;
-
-	if (i+1 >= total) {
-		fprintf(stderr, "\r%s100%%\n", header);
-		return;
-	}
-
-	if (total / 200 == 0)
-		divider = 1;
-
-	/* only bother updating about every 0.5% */
-	if (i % (total / divider) == 0 || i+1 >= total) {
-		fprintf(stderr, "\r%s%3llu%%", header,
-			(long long unsigned) i * 100 / total);
-	}
-}
-
-/* mappable aperture trasher helper */
-drm_intel_bo **trash_bos;
-int num_trash_bos;
-
-void igt_init_aperture_trashers(drm_intel_bufmgr *bufmgr)
-{
-	int i;
-
-	num_trash_bos = gem_mappable_aperture_size() / (1024*1024);
-
-	trash_bos = malloc(num_trash_bos * sizeof(drm_intel_bo *));
-	assert(trash_bos);
-
-	for (i = 0; i < num_trash_bos; i++)
-		trash_bos[i] = drm_intel_bo_alloc(bufmgr, "trash bo", 1024*1024, 4096);
-}
-
-void igt_trash_aperture(void)
-{
-	int i;
-	uint8_t *gtt_ptr;
-
-	for (i = 0; i < num_trash_bos; i++) {
-		drm_intel_gem_bo_map_gtt(trash_bos[i]);
-		gtt_ptr = trash_bos[i]->virtual;
-		*gtt_ptr = 0;
-		drm_intel_gem_bo_unmap_gtt(trash_bos[i]);
-	}
-}
-
-void igt_cleanup_aperture_trashers(void)
-{
-	int i;
-
-	for (i = 0; i < num_trash_bos; i++)
-		drm_intel_bo_unreference(trash_bos[i]);
-
-	free(trash_bos);
-}
-
-void igt_system_suspend_autoresume(void)
-{
-	int ret;
-
-	/* FIXME: Simulation doesn't like suspend/resume, and not even a lighter
-	 * approach using /sys/power/pm_test to just test our driver's callbacks
-	 * seems to fare better. We need to investigate what's going on. */
-	igt_skip_on_simulation();
-
-	ret = system("rtcwake -s 30 -m mem");
-	igt_assert(ret == 0);
-}
-
-void igt_drop_root(void)
-{
-	igt_assert(getuid() == 0);
-
-	igt_assert(setgid(2) == 0);
-	igt_assert(setuid(2) == 0);
-
-	igt_assert(getgid() == 2);
-	igt_assert(getuid() == 2);
-}
-
-void igt_wait_for_keypress(void)
-{
-	struct termios oldt, newt;
-
-	if (!isatty(STDIN_FILENO))
-		return;
-
-	tcgetattr ( STDIN_FILENO, &oldt );
-	newt = oldt;
-	newt.c_lflag &= ~( ICANON | ECHO );
-	tcsetattr ( STDIN_FILENO, TCSANOW, &newt );
-	getchar();
-	tcsetattr ( STDIN_FILENO, TCSANOW, &oldt );
-}
