@@ -29,6 +29,7 @@
 #define IGT_AUX_H
 
 #include <intel_bufmgr.h>
+#include <stdbool.h>
 
 /* auxialiary igt helpers from igt_aux.c */
 /* generally useful helpers */
@@ -55,7 +56,7 @@ void igt_system_suspend_autoresume(void);
 /* dropping priviledges */
 void igt_drop_root(void);
 
-void igt_wait_for_keypress(void);
+void igt_debug_wait_for_keypress(const char *key);
 
 enum igt_runtime_pm_status {
 	IGT_RUNTIME_PM_STATUS_ACTIVE,
