@@ -102,7 +102,7 @@ static bool __igt_debugfs_init(igt_debugfs_t *debugfs)
 
 	igt_assert(stat("/sys/kernel/debug", &st) == 0);
 
-	igt_assert(mount("debug", "/sys/kernel/debug", "debugfs", 0, 0) == 0);
+	mount("debug", "/sys/kernel/debug", "debugfs", 0, 0);
 
 find_minor:
 	strcpy(debugfs->root, path);
@@ -199,9 +199,14 @@ bool igt_crc_is_null(igt_crc_t *crc)
 {
 	int i;
 
-	for (i = 0; i < crc->n_words; i++)
+	for (i = 0; i < crc->n_words; i++) {
+		igt_warn_on_f(crc->crc[i] == 0xffffffff,
+			      "Suspicious CRC: it looks like the CRC "
+			      "read back was from a register in a powered "
+			      "down well\n");
 		if (crc->crc[i])
 			return false;
+	}
 
 	return true;
 }
@@ -289,7 +294,7 @@ static bool igt_pipe_crc_do_start(igt_pipe_crc_t *pipe_crc)
 {
 	char buf[64];
 
-	sprintf(buf, "pipe %c %s", pipe_name(pipe_crc->pipe),
+	sprintf(buf, "pipe %s %s", kmstest_pipe_name(pipe_crc->pipe),
 		pipe_crc_source_name(pipe_crc->source));
 	errno = 0;
 	write(pipe_crc->ctl_fd, buf, strlen(buf));
@@ -303,7 +308,7 @@ static void igt_pipe_crc_pipe_off(int fd, enum pipe pipe)
 {
 	char buf[32];
 
-	sprintf(buf, "pipe %c none", pipe_name(pipe));
+	sprintf(buf, "pipe %s none", kmstest_pipe_name(pipe));
 	write(fd, buf, strlen(buf));
 }
 
@@ -374,7 +379,7 @@ igt_pipe_crc_new(enum pipe pipe, enum intel_pipe_crc_source source)
 	pipe_crc->ctl_fd = igt_debugfs_open("i915_display_crc_ctl", O_WRONLY);
 	igt_assert(pipe_crc->ctl_fd != -1);
 
-	sprintf(buf, "i915_pipe_%c_crc", pipe_name(pipe));
+	sprintf(buf, "i915_pipe_%s_crc", kmstest_pipe_name(pipe));
 	pipe_crc->crc_fd = igt_debugfs_open(buf, O_RDONLY);
 	igt_assert(pipe_crc->crc_fd != -1);
 
@@ -440,7 +445,7 @@ void igt_pipe_crc_stop(igt_pipe_crc_t *pipe_crc)
 {
 	char buf[32];
 
-	sprintf(buf, "pipe %c none", pipe_name(pipe_crc->pipe));
+	sprintf(buf, "pipe %s none", kmstest_pipe_name(pipe_crc->pipe));
 	write(pipe_crc->ctl_fd, buf, strlen(buf));
 }
 
@@ -463,7 +468,7 @@ static bool read_one_crc(igt_pipe_crc_t *pipe_crc, igt_crc_t *out)
 	bytes_read = read(pipe_crc->crc_fd, &buf, pipe_crc->line_len);
 	igt_set_timeout(0);
 
-	igt_assert_cmpint(bytes_read, ==, pipe_crc->line_len);
+	igt_assert_eq(bytes_read, pipe_crc->line_len);
 	buf[bytes_read] = '\0';
 
 	if (!pipe_crc_init_from_string(out, buf))
@@ -522,6 +527,8 @@ void igt_pipe_crc_collect_crc(igt_pipe_crc_t *pipe_crc, igt_crc_t *out_crc)
 	igt_pipe_crc_start(pipe_crc);
 	read_one_crc(pipe_crc, out_crc);
 	igt_pipe_crc_stop(pipe_crc);
+
+	igt_assert(!igt_crc_is_null(out_crc));
 }
 
 /*
@@ -618,6 +625,8 @@ void igt_enable_prefault(void)
  */
 int igt_open_forcewake_handle(void)
 {
+	if (getenv("IGT_NO_FORCEWAKE"))
+		return -1;
 	return igt_debugfs_open("i915_forcewake_user", O_WRONLY);
 }
 
@@ -722,7 +731,7 @@ void igt_set_stop_rings(enum stop_ring_flags flags)
 
 	stop_rings_write(flags);
 	current = igt_get_stop_rings();
-	if (current != flags)
-		igt_warn("i915_ring_stop readback mismatch 0x%x vs 0x%x\n",
-			 flags, current);
+	igt_warn_on_f(current != flags,
+		      "i915_ring_stop readback mismatch 0x%x vs 0x%x\n",
+		      flags, current);
 }
