@@ -12,11 +12,12 @@
 struct intel_batchbuffer {
 	drm_intel_bufmgr *bufmgr;
 	uint32_t devid;
+	int gen;
 
 	drm_intel_bo *bo;
 
 	uint8_t buffer[BATCH_SZ];
-	uint8_t *ptr;
+	uint8_t *ptr, *end;
 	uint8_t *state;
 };
 
@@ -38,7 +39,7 @@ void intel_batchbuffer_data(struct intel_batchbuffer *batch,
 
 void intel_batchbuffer_emit_reloc(struct intel_batchbuffer *batch,
 				  drm_intel_bo *buffer,
-				  uint32_t delta,
+				  uint64_t delta,
 				  uint32_t read_domains,
 				  uint32_t write_domain,
 				  int fenced);
@@ -76,6 +77,7 @@ intel_batchbuffer_require_space(struct intel_batchbuffer *batch,
 /**
  * BEGIN_BATCH:
  * @n: number of DWORDS to emit
+ * @r: number of RELOCS to emit
  *
  * Prepares a batch to emit @n DWORDS, flushing it if there's not enough space
  * available.
@@ -83,8 +85,13 @@ intel_batchbuffer_require_space(struct intel_batchbuffer *batch,
  * This macro needs a pointer to an #intel_batchbuffer structure called batch in
  * scope.
  */
-#define BEGIN_BATCH(n) do {						\
-	intel_batchbuffer_require_space(batch, (n)*4);			\
+#define BEGIN_BATCH(n, r) do {						\
+	int __n = (n); \
+	igt_assert(batch->end == NULL); \
+	if (batch->gen >= 8) __n += r;	\
+	__n *= 4; \
+	intel_batchbuffer_require_space(batch, __n);			\
+	batch->end = batch->ptr + __n; \
 } while (0)
 
 /**
@@ -143,49 +150,25 @@ intel_batchbuffer_require_space(struct intel_batchbuffer *batch,
  * scope.
  */
 #define ADVANCE_BATCH() do {						\
+	igt_assert(batch->ptr == batch->end); \
+	batch->end = NULL; \
 } while(0)
 
-#define BLIT_COPY_BATCH_START(devid, flags) do { \
-	if (intel_gen(devid) >= 8) { \
-		BEGIN_BATCH(10); \
-		OUT_BATCH(XY_SRC_COPY_BLT_CMD | \
-				XY_SRC_COPY_BLT_WRITE_ALPHA | \
-				XY_SRC_COPY_BLT_WRITE_RGB | \
-				(flags) | 8); \
-	} else { \
-		BEGIN_BATCH(8); \
-		OUT_BATCH(XY_SRC_COPY_BLT_CMD | \
-				XY_SRC_COPY_BLT_WRITE_ALPHA | \
-				XY_SRC_COPY_BLT_WRITE_RGB | \
-				(flags) | 6); \
-	} \
+#define BLIT_COPY_BATCH_START(flags) do { \
+	BEGIN_BATCH(8, 2); \
+	OUT_BATCH(XY_SRC_COPY_BLT_CMD | \
+		  XY_SRC_COPY_BLT_WRITE_ALPHA | \
+		  XY_SRC_COPY_BLT_WRITE_RGB | \
+		  (flags) | \
+		  (6 + 2*(batch->gen >= 8))); \
 } while(0)
 
-#define COLOR_BLIT_COPY_BATCH_START(devid, flags) do { \
-	if (intel_gen(devid) >= 8) { \
-		BEGIN_BATCH(8); \
-		OUT_BATCH(MI_NOOP); \
-		OUT_BATCH(XY_COLOR_BLT_CMD_NOLEN | 0x5 | \
-				COLOR_BLT_WRITE_ALPHA | \
-				XY_COLOR_BLT_WRITE_RGB); \
-	} else { \
-		BEGIN_BATCH(6); \
-		OUT_BATCH(XY_COLOR_BLT_CMD_NOLEN | 0x4 | \
-				COLOR_BLT_WRITE_ALPHA | \
-				XY_COLOR_BLT_WRITE_RGB); \
-	} \
-} while(0)
-
-/**
- * BLIT_RELOC_UDW:
- * @devid: pci device id of the drm device
- *
- * Emits the upper relocation DWORD on gen8+ and nothing on earlier generations.
- */
-#define BLIT_RELOC_UDW(devid) do { \
-	if (intel_gen(devid) >= 8) { \
-		OUT_BATCH(0); \
-	} \
+#define COLOR_BLIT_COPY_BATCH_START(flags) do { \
+	BEGIN_BATCH(6, 1); \
+	OUT_BATCH(XY_COLOR_BLT_CMD_NOLEN | \
+		  COLOR_BLT_WRITE_ALPHA | \
+		  XY_COLOR_BLT_WRITE_RGB | \
+		  (4 + (batch->gen >= 8))); \
 } while(0)
 
 void
@@ -204,14 +187,10 @@ void intel_copy_bo(struct intel_batchbuffer *batch,
  * @tiling: tiling mode bits
  * @data: pointer to the memory mapping of the buffer
  * @size: size of the buffer object
- * @num_tiles: number of tiles of the buffer object
  *
  * This is a i-g-t buffer object wrapper structure which augments the baseline
  * libdrm buffer object with suitable data needed by the render copy and the
  * media fill functions.
- *
- * Note that @num_tiles is only used by gem_stress.c internally and can be
- * ignored.
  */
 struct igt_buf {
     drm_intel_bo *bo;
@@ -219,6 +198,7 @@ struct igt_buf {
     uint32_t tiling;
     uint32_t *data;
     uint32_t size;
+    /*< private >*/
     unsigned num_tiles;
 };
 
