@@ -1,4 +1,17 @@
 #!/bin/sh
+
+# hacked-up long option parsing
+for arg in $@ ; do
+	case $arg in
+		--list-subtests)
+			exit 79
+			;;
+		--run-subtest)
+			exit 79
+			;;
+	esac
+done
+
 die() {
 	echo "$@"
 	exit 1
@@ -34,6 +47,8 @@ if [ `cat $i915_dfs_path/clients | wc -l` -gt "2" ] ; then
 		die "ERROR: other drm clients running"
 fi
 
+whoami | grep -q root || ( echo ERROR: not running as root; exit 1 )
+
 i915_sfs_path=
 if [ -d /sys/class/drm ] ; then
     sysfs_path=/sys/class/drm
