@@ -37,10 +37,11 @@
 #include <sys/ioctl.h>
 #include <sys/time.h>
 #include "drm.h"
-#include "i915_drm.h"
+#include "ioctl_wrappers.h"
 #include "drmtest.h"
 #include "intel_chipset.h"
-#include "intel_gpu_tools.h"
+#include "intel_io.h"
+#include "igt_debugfs.h"
 
 /* Testcase: Submit patches with relocations in memory that will fault
  *
@@ -49,12 +50,14 @@
 
 #define OBJECT_SIZE 16384
 
-#define COPY_BLT_CMD		(2<<29|0x53<<22|0x6)
+#define COPY_BLT_CMD_NOLEN	(2<<29|0x53<<22)
 #define BLT_WRITE_ALPHA		(1<<21)
 #define BLT_WRITE_RGB		(1<<20)
 #define BLT_SRC_TILED		(1<<15)
 #define BLT_DST_TILED		(1<<11)
 
+uint32_t devid;
+
 static int gem_linear_blt(uint32_t *batch,
 			  uint32_t src,
 			  uint32_t dst,
@@ -68,7 +71,11 @@ static int gem_linear_blt(uint32_t *batch,
 
 	if (height) {
 		int i = 0;
-		b[i++] = COPY_BLT_CMD | BLT_WRITE_ALPHA | BLT_WRITE_RGB;
+		b[i++] = COPY_BLT_CMD_NOLEN | BLT_WRITE_ALPHA | BLT_WRITE_RGB;
+		if (intel_gen(devid) >= 8)
+			b[i-1] |= 8;
+		else
+			b[i-1] |= 6;
 		b[i++] = 0xcc << 16 | 1 << 25 | 1 << 24 | (16*1024);
 		b[i++] = 0;
 		b[i++] = height << 16 | (4*1024);
@@ -81,10 +88,17 @@ static int gem_linear_blt(uint32_t *batch,
 		reloc->presumed_offset = 0;
 		reloc++;
 
+		if (intel_gen(devid) >= 8)
+			b[i++] = 0; /* FIXME: use real high dword */
+
 		b[i++] = 0;
 		b[i++] = 16*1024;
 		b[i++] = 0;
 		reloc->offset = (b-batch+7) * sizeof(uint32_t);
+		if (intel_gen(devid) >= 8) {
+			reloc->offset += sizeof(uint32_t);
+			b[i++] = 0; /* FIXME: use real high dword */
+		}
 		reloc->delta = 0;
 		reloc->target_handle = src;
 		reloc->read_domains = I915_GEM_DOMAIN_RENDER;
@@ -92,13 +106,20 @@ static int gem_linear_blt(uint32_t *batch,
 		reloc->presumed_offset = 0;
 		reloc++;
 
-		b += 8;
+		if (intel_gen(devid) >= 8)
+			b += 10;
+		else
+			b += 8;
 		length -= height * 16*1024;
 	}
 	
 	if (length) {
 		int i = 0;
-		b[i++] = COPY_BLT_CMD | BLT_WRITE_ALPHA | BLT_WRITE_RGB;
+		b[i++] = COPY_BLT_CMD_NOLEN | BLT_WRITE_ALPHA | BLT_WRITE_RGB;
+		if (intel_gen(devid) >= 8)
+			b[i-1] |= 8;
+		else
+			b[i-1] |= 6;
 		b[i++] = 0xcc << 16 | 1 << 25 | 1 << 24 | (16*1024);
 		b[i++] = height << 16;
 		b[i++] = (1+height) << 16 | (length / 4);
@@ -110,11 +131,17 @@ static int gem_linear_blt(uint32_t *batch,
 		reloc->write_domain = I915_GEM_DOMAIN_RENDER;
 		reloc->presumed_offset = 0;
 		reloc++;
+		if (intel_gen(devid) >= 8)
+			b[i++] = 0; /* FIXME: use real high dword */
 
 		b[i++] = height << 16;
 		b[i++] = 16*1024;
 		b[i++] = 0;
 		reloc->offset = (b-batch+7) * sizeof(uint32_t);
+		if (intel_gen(devid) >= 8) {
+			reloc->offset += sizeof(uint32_t);
+			b[i++] = 0; /* FIXME: use real high dword */
+		}
 		reloc->delta = 0;
 		reloc->target_handle = src;
 		reloc->read_domains = I915_GEM_DOMAIN_RENDER;
@@ -122,7 +149,10 @@ static int gem_linear_blt(uint32_t *batch,
 		reloc->presumed_offset = 0;
 		reloc++;
 
-		b += 8;
+		if (intel_gen(devid) >= 8)
+			b += 10;
+		else
+			b += 8;
 	}
 
 	b[0] = MI_BATCH_BUFFER_END;
@@ -136,13 +166,14 @@ static void run(int object_size)
 	struct drm_i915_gem_execbuffer2 execbuf;
 	struct drm_i915_gem_exec_object2 exec[3];
 	struct drm_i915_gem_relocation_entry reloc[4];
-	uint32_t buf[20];
+	uint32_t buf[40];
 	uint32_t handle, handle_relocs, src, dst;
 	void *gtt_relocs;
 	int fd, len;
 	int ring;
 
 	fd = drm_open_any();
+	devid = intel_get_drm_devid(fd);
 	handle = gem_create(fd, 4096);
 	src = gem_create(fd, object_size);
 	dst = gem_create(fd, object_size);
@@ -175,7 +206,10 @@ static void run(int object_size)
 	igt_assert(gtt_relocs);
 
 	exec[2].handle = handle;
-	exec[2].relocation_count = len > 40 ? 4 : 2;
+	if (intel_gen(devid) >= 8)
+		exec[2].relocation_count = len > 56 ? 4 : 2;
+	else
+		exec[2].relocation_count = len > 40 ? 4 : 2;
 	/* A newly mmap gtt bo will fault on first access. */
 	exec[2].relocs_ptr = (uintptr_t)gtt_relocs;
 	exec[2].alignment = 0;
@@ -185,7 +219,7 @@ static void run(int object_size)
 	exec[2].rsvd2 = 0;
 
 	ring = 0;
-	if (HAS_BLT_RING(intel_get_drm_devid(fd)))
+	if (HAS_BLT_RING(devid))
 		ring = I915_EXEC_BLT;
 
 	execbuf.buffers_ptr = (uintptr_t)exec;
