@@ -46,7 +46,6 @@
 #include <errno.h>
 #include <sys/stat.h>
 #include <sys/time.h>
-#include <getopt.h>
 
 #include <drm.h>
 
@@ -90,7 +89,7 @@ int main(int argc, char **argv)
 	uint32_t start = 0;
 	int i, j, fd, count;
 
-	igt_simple_init();
+	igt_simple_init(argc, argv);
 
 	fd = drm_open_any();
 
@@ -109,7 +108,7 @@ int main(int argc, char **argv)
 	if (count == 0)
 		count = 3 * gem_aperture_size(fd) / SIZE / 2;
 	else if (count < 2) {
-		fprintf(stderr, "count must be >= 2\n");
+		igt_warn("count must be >= 2\n");
 		return 1;
 	}
 
