@@ -126,7 +126,7 @@ static void bo_check(data_t *data, drm_intel_bo *bo, uint32_t val)
 	gem_read(data->drm_fd, bo->handle, 0,
 		 data->linear, sizeof(data->linear));
 	for (i = 0; i < WIDTH * HEIGHT; i++)
-		igt_assert_cmpint(data->linear[i], ==, val);
+		igt_assert_eq_u32(data->linear[i], val);
 }
 
 static void scratch_buf_init_from_bo(struct igt_buf *buf, drm_intel_bo *bo)
@@ -310,7 +310,7 @@ static void run_test(data_t *data, enum ring r1, enum ring r2, enum test test)
 		bo_check(data, b, 0xc);
 		break;
 	default:
-		abort();
+		igt_fail(1);
 	}
 
 	r1_ops->busy_fini(data);
