@@ -338,7 +338,7 @@ int main(int argc, char **argv)
 	uint32_t start = 0;
 	int i, fd, count;
 
-	igt_simple_init();
+	igt_simple_init(argc, argv);
 
 	fd = drm_open_any();
 
