@@ -179,22 +179,20 @@ static void emit_dummy_load__bcs(struct test_output *o)
 	igt_assert(target_bo);
 
 	for (i = 0; i < limit; i++) {
-		BLIT_COPY_BATCH_START(devid, 0);
+		BLIT_COPY_BATCH_START(0);
 		OUT_BATCH((3 << 24) | /* 32 bits */
 			  (0xcc << 16) | /* copy ROP */
 			  pitch);
 		OUT_BATCH(0 << 16 | 0);
 		OUT_BATCH(o->fb_height << 16 | o->fb_width);
 		OUT_RELOC_FENCED(dummy_bo, I915_GEM_DOMAIN_RENDER, I915_GEM_DOMAIN_RENDER, 0);
-		BLIT_RELOC_UDW(devid);
 		OUT_BATCH(0 << 16 | 0);
 		OUT_BATCH(pitch);
 		OUT_RELOC_FENCED(target_bo, I915_GEM_DOMAIN_RENDER, 0, 0);
-		BLIT_RELOC_UDW(devid);
 		ADVANCE_BATCH();
 
-		if (IS_GEN6(devid) || IS_GEN7(devid)) {
-			BEGIN_BATCH(3);
+		if (batch->gen >= 6) {
+			BEGIN_BATCH(3, 0);
 			OUT_BATCH(XY_SETUP_CLIP_BLT_CMD);
 			OUT_BATCH(0);
 			OUT_BATCH(0);
@@ -227,7 +225,7 @@ static void emit_fence_stress(struct test_output *o)
 		uint32_t tiling = I915_TILING_X;
 		unsigned long pitch = 0;
 		bo[i] = drm_intel_bo_alloc_tiled(bufmgr,
-						 "tiled bo", 1024, 1024, 4,
+						 "X tiled bo", 1024, 1024, 4,
 						 &tiling, &pitch, 0);
 		exec[i].handle = bo[i]->handle;
 		exec[i].flags = EXEC_OBJECT_NEEDS_FENCE;
@@ -989,8 +987,8 @@ static void connector_find_preferred_mode(uint32_t connector_id, int crtc_idx,
 {
 	struct kmstest_connector_config config;
 
-	if (kmstest_get_connector_config(drm_fd, connector_id, 1 << crtc_idx,
-					 &config) < 0) {
+	if (!kmstest_get_connector_config(drm_fd, connector_id, 1 << crtc_idx,
+					  &config)) {
 		o->mode_valid = 0;
 		return;
 	}
@@ -1032,12 +1030,12 @@ static void connector_find_compatible_mode(int crtc_idx0, int crtc_idx1,
 	drmModeModeInfo *mode[2];
 	int n, m;
 
-	if (kmstest_get_connector_config(drm_fd, o->_connector[0],
-					 1 << crtc_idx0, &config[0]) < 0)
+	if (!kmstest_get_connector_config(drm_fd, o->_connector[0],
+					  1 << crtc_idx0, &config[0]))
 		return;
 
-	if (kmstest_get_connector_config(drm_fd, o->_connector[1],
-					 1 << crtc_idx1, &config[1]) < 0) {
+	if (!kmstest_get_connector_config(drm_fd, o->_connector[1],
+					  1 << crtc_idx1, &config[1])) {
 		kmstest_free_connector_config(&config[0]);
 		return;
 	}
@@ -1241,7 +1239,7 @@ static void run_test_on_crtc_set(struct test_output *o, int *crtc_idxs,
 	char test_name[128];
 	unsigned elapsed;
 	unsigned bo_size = 0;
-	bool tiled;
+	unsigned int tiling;
 	int i;
 
 	switch (crtc_count) {
@@ -1273,9 +1271,9 @@ static void run_test_on_crtc_set(struct test_output *o, int *crtc_idxs,
 	if (o->flags & TEST_PAN)
 		o->fb_width *= 2;
 
-	tiled = false;
+	tiling = I915_TILING_NONE;
 	if (o->flags & TEST_FENCE_STRESS)
-		tiled = true;
+		tiling = I915_TILING_X;
 
 	/* 256 MB is usually the maximum mappable aperture,
 	 * (make it 4x times that to ensure failure) */
@@ -1284,13 +1282,13 @@ static void run_test_on_crtc_set(struct test_output *o, int *crtc_idxs,
 
 	o->fb_ids[0] = igt_create_fb(drm_fd, o->fb_width, o->fb_height,
 					 igt_bpp_depth_to_drm_format(o->bpp, o->depth),
-					 tiled, &o->fb_info[0]);
+					 tiling, &o->fb_info[0]);
 	o->fb_ids[1] = igt_create_fb_with_bo_size(drm_fd, o->fb_width, o->fb_height,
 					 igt_bpp_depth_to_drm_format(o->bpp, o->depth),
-					 tiled, &o->fb_info[1], bo_size);
+					 tiling, &o->fb_info[1], bo_size);
 	o->fb_ids[2] = igt_create_fb(drm_fd, o->fb_width, o->fb_height,
 					 igt_bpp_depth_to_drm_format(o->bpp, o->depth),
-					 true, &o->fb_info[2]);
+					 I915_TILING_X, &o->fb_info[2]);
 	igt_assert(o->fb_ids[0]);
 	igt_assert(o->fb_ids[1]);
 	if (o->flags & TEST_FB_BAD_TILING)
@@ -1370,7 +1368,6 @@ static int run_test(int duration, int flags)
 
 	igt_require((flags & TEST_HANG) == 0 || !is_hung(drm_fd));
 
-
 	if (flags & TEST_RPM)
 		igt_require(igt_setup_runtime_pm());
 
@@ -1397,7 +1394,7 @@ static int run_test(int duration, int flags)
 		}
 	}
 
-	igt_assert(modes);
+	igt_require(modes);
 	duration = duration * 1000 / modes;
 	duration = duration < 500 ? 500 : duration;
 
@@ -1595,7 +1592,9 @@ int main(int argc, char **argv)
 	igt_fixture {
 		drm_fd = drm_open_any();
 
-		igt_set_vt_graphics_mode();
+		igt_enable_connectors();
+
+		kmstest_set_vt_graphics_mode();
 		igt_install_exit_handler(kms_flip_exit_handler);
 		get_timestamp_format();
 
