@@ -39,6 +39,14 @@ typedef struct {
 	struct igt_fb fb;
 } data_t;
 
+static struct {
+	double r, g, b;
+	igt_crc_t crc;
+} colors[2] = {
+	{ .r = 0.0, .g = 1.0, .b = 0.0 },
+	{ .r = 0.0, .g = 1.0, .b = 1.0 },
+};
+
 static uint64_t submit_batch(int fd, unsigned ring_id)
 {
 	const uint32_t batch[] = { MI_NOOP,
@@ -96,41 +104,44 @@ static void test_bad_command(data_t *data, const char *cmd)
 	ctl = igt_debugfs_fopen("i915_display_crc_ctl", "r+");
 	written = fwrite(cmd, 1, strlen(cmd), ctl);
 	fflush(ctl);
-	igt_assert_cmpint(written, ==, (strlen(cmd)));
+	igt_assert_eq(written, strlen(cmd));
 	igt_assert(ferror(ctl));
-	igt_assert_cmpint(errno, ==, EINVAL);
+	igt_assert_eq(errno, EINVAL);
 
 	fclose(ctl);
 }
 
+#define N_CRCS	3
+
 #define TEST_SEQUENCE (1<<0)
 
-static void test_read_crc(data_t *data, int pipe, unsigned flags)
+static int
+test_read_crc_for_output(data_t *data, int pipe, igt_output_t *output,
+			 unsigned flags)
 {
 	igt_display_t *display = &data->display;
+	igt_plane_t *primary;
+	drmModeModeInfo *mode;
 	igt_pipe_crc_t *pipe_crc;
 	igt_crc_t *crcs = NULL;
-	int valid_connectors = 0;
-	igt_output_t *output;
+	int c, j;
 
-	igt_skip_on(pipe >= data->display.n_pipes);
-
-	for_each_connected_output(display, output) {
-		igt_plane_t *primary;
-		drmModeModeInfo *mode;
+	for (c = 0; c < ARRAY_SIZE(colors); c++) {
+		char *crc_str;
 
 		igt_output_set_pipe(output, pipe);
 
-		igt_info("%s: Testing connector %s using pipe %c\n",
-			 igt_subtest_name(), igt_output_name(output),
-			 pipe_name(pipe));
+		igt_debug("Clearing the fb with color (%.02lf,%.02lf,%.02lf)\n",
+			  colors[c].r, colors[c].g, colors[c].b);
 
 		mode = igt_output_get_mode(output);
 		igt_create_color_fb(data->drm_fd,
 					mode->hdisplay, mode->vdisplay,
 					DRM_FORMAT_XRGB8888,
-					false, /* tiled */
-					0.0, 1.0, 0.0,
+					I915_TILING_NONE,
+					colors[c].r,
+					colors[c].g,
+					colors[c].b,
 					&data->fb);
 
 		primary = igt_output_get_plane(output, 0);
@@ -141,29 +152,44 @@ static void test_read_crc(data_t *data, int pipe, unsigned flags)
 		pipe_crc = igt_pipe_crc_new(pipe, INTEL_PIPE_CRC_SOURCE_AUTO);
 
 		if (!pipe_crc)
-			continue;
-		valid_connectors++;
+			return 0;
 
 		igt_pipe_crc_start(pipe_crc);
 
-		/* wait for 3 vblanks and the corresponding 3 CRCs */
-		igt_pipe_crc_get_crcs(pipe_crc, 3, &crcs);
+		/* wait for N_CRCS vblanks and the corresponding N_CRCS CRCs */
+		igt_pipe_crc_get_crcs(pipe_crc, N_CRCS, &crcs);
 
 		igt_pipe_crc_stop(pipe_crc);
 
+		/*
+		 * save the CRC in colors so it can be compared to the CRC of
+		 * other fbs
+		 */
+		colors[c].crc = crcs[0];
+
+		crc_str = igt_crc_to_string(&crcs[0]);
+		igt_debug("CRC for this fb: %s\n", crc_str);
+		free(crc_str);
+
+		/*
+		 * make sure the CRC of this fb is different from the ones of
+		 * previous fbs
+		 */
+		for (j = 0; j < c; j++)
+			igt_assert(!igt_crc_equal(&colors[j].crc,
+						  &colors[c].crc));
+
 		/* ensure the CRCs are not all 0s */
-		igt_assert(!igt_crc_is_null(&crcs[0]));
-		igt_assert(!igt_crc_is_null(&crcs[1]));
-		igt_assert(!igt_crc_is_null(&crcs[2]));
+		for (j = 0; j < N_CRCS; j++)
+			igt_assert(!igt_crc_is_null(&crcs[j]));
 
 		/* and ensure that they'are all equal, we haven't changed the fb */
-		igt_assert(igt_crc_equal(&crcs[0], &crcs[1]));
-		igt_assert(igt_crc_equal(&crcs[1], &crcs[2]));
+		for (j = 0; j < (N_CRCS - 1); j++)
+			igt_assert(igt_crc_equal(&crcs[j], &crcs[j + 1]));
 
-		if (flags & TEST_SEQUENCE) {
-			igt_assert(crcs[0].frame + 1 == crcs[1].frame);
-			igt_assert(crcs[1].frame + 1 == crcs[2].frame);
-		}
+		if (flags & TEST_SEQUENCE)
+			for (j = 0; j < (N_CRCS - 1); j++)
+				igt_assert(crcs[j].frame + 1 == crcs[j + 1].frame);
 
 		free(crcs);
 		igt_pipe_crc_free(pipe_crc);
@@ -173,8 +199,27 @@ static void test_read_crc(data_t *data, int pipe, unsigned flags)
 		igt_output_set_pipe(output, PIPE_ANY);
 	}
 
-	igt_require_f(valid_connectors, "No connector found for pipe %i\n", pipe);
+	return 1;
+}
 
+static void test_read_crc(data_t *data, int pipe, unsigned flags)
+{
+	igt_display_t *display = &data->display;
+	int valid_connectors = 0;
+	igt_output_t *output;
+
+	igt_skip_on(pipe >= data->display.n_pipes);
+
+	for_each_connected_output(display, output) {
+
+		igt_info("%s: Testing connector %s using pipe %s\n",
+			 igt_subtest_name(), igt_output_name(output),
+			 kmstest_pipe_name(pipe));
+
+		valid_connectors += test_read_crc_for_output(data, pipe, output, flags);
+	}
+
+	igt_require_f(valid_connectors, "No connector found for pipe %i\n", pipe);
 }
 
 data_t data = {0, };
@@ -186,7 +231,9 @@ igt_main
 	igt_fixture {
 		data.drm_fd = drm_open_any();
 
-		igt_set_vt_graphics_mode();
+		igt_enable_connectors();
+
+		kmstest_set_vt_graphics_mode();
 
 		igt_require_pipe_crc();
 
