@@ -29,7 +29,6 @@
 #include <stdint.h>
 #include <unistd.h>
 #include <string.h>
-#include <getopt.h>
 #include <sys/time.h>
 
 #include "drmtest.h"
@@ -180,16 +179,14 @@ static void create_fb_for_crtc(struct crtc_config *crtc,
 {
 	int bpp;
 	int depth;
-	bool enable_tiling;
 	int fb_id;
 
 	bpp = 32;
 	depth = 24;
-	enable_tiling = false;
 	fb_id = igt_create_fb(drm_fd, crtc->mode.hdisplay,
 				  crtc->mode.vdisplay,
 				  igt_bpp_depth_to_drm_format(bpp, depth),
-				  enable_tiling, fb_info);
+				  I915_TILING_NONE, fb_info);
 	igt_assert(fb_id > 0);
 }
 
@@ -247,7 +244,7 @@ static void get_crtc_config_str(struct crtc_config *crtc, char *buf,
 
 	pos = snprintf(buf, buf_size,
 		       "CRTC[%d] [Pipe %s] Mode: %s@%dHz Connectors: ",
-		       crtc->crtc_id, kmstest_pipe_str(crtc->pipe_id),
+		       crtc->crtc_id, kmstest_pipe_name(crtc->pipe_id),
 		       crtc->mode.name, crtc->mode.vrefresh);
 	if (pos > buf_size)
 		return;
@@ -518,8 +515,8 @@ static int get_one_connector(drmModeRes *resources, int connector_id,
 	}
 
 	if (cconf->connected) {
-		if (kmstest_get_connector_default_mode(drm_fd, connector,
-							&mode) < 0)
+		if (!kmstest_get_connector_default_mode(drm_fd, connector,
+							&mode))
 			mode = mode_640_480;
 	} else {
 		mode = mode_640_480;
@@ -715,7 +712,7 @@ int main(int argc, char **argv)
 	igt_fixture {
 		drm_fd = drm_open_any();
 		if (!dry_run)
-			igt_set_vt_graphics_mode();
+			kmstest_set_vt_graphics_mode();
 
 		drm_resources = drmModeGetResources(drm_fd);
 		igt_assert(drm_resources);
