@@ -22,11 +22,12 @@
 #include <sys/stat.h>
 #include <sys/ioctl.h>
 
-#include "i915_drm.h"
+#include "ioctl_wrappers.h"
 #include "intel_bufmgr.h"
 #include "nouveau.h"
-#include "intel_gpu_tools.h"
+#include "intel_io.h"
 #include "intel_batchbuffer.h"
+#include "intel_chipset.h"
 #include "drmtest.h"
 
 int intel_fd = -1, nouveau_fd = -1;
@@ -336,7 +337,7 @@ static void test_i915_blt_fill_nv_read(void)
 	igt_assert(nouveau_bo_prime_handle_ref(ndev, prime_fd, &nvbo) == 0);
 	close(prime_fd);
 
-	intel_copy_bo(intel_batch, test_intel_bo, src_bo, 256, 1);
+	intel_copy_bo(intel_batch, test_intel_bo, src_bo, BO_SIZE);
 
 	igt_assert(nouveau_bo_map(nvbo, NOUVEAU_BO_RDWR, nclient) == 0);
 
