@@ -30,7 +30,8 @@
 #include <stdio.h>
 #include <err.h>
 #include <string.h>
-#include "intel_gpu_tools.h"
+#include "intel_io.h"
+#include "intel_chipset.h"
 
 static void usage(char *cmdname)
 {
