@@ -53,8 +53,9 @@
 #include <intel_bufmgr.h>
 
 #include "intel_chipset.h"
-#include "intel_gpu_tools.h"
+#include "intel_io.h"
 #include "instdone.h"
+#include "intel_reg.h"
 
 static uint32_t
 print_head(unsigned int reg)
