@@ -33,7 +33,10 @@
 #include <string.h>
 #include <err.h>
 #include <unistd.h>
-#include "intel_gpu_tools.h"
+#include "intel_io.h"
+#include "intel_chipset.h"
+#include "intel_reg.h"
+#include "drmtest.h"
 
 static uint32_t devid = 0;
 
@@ -2928,21 +2931,21 @@ int main(int argc, char** argv)
 	}
 
 	if (file) {
-		intel_map_file(file);
+		intel_mmio_use_dump_file(file);
 		if (devid) {
 			if (IS_GEN5(devid))
-				pch = PCH_IBX;
+				intel_pch = PCH_IBX;
 			else if (IS_GEN6(devid) || IS_IVYBRIDGE(devid))
-				pch = PCH_CPT;
+				intel_pch = PCH_CPT;
 			else if (IS_HASWELL(devid))
-				pch = PCH_LPT;
+				intel_pch = PCH_LPT;
 			else
-				pch = PCH_NONE;
+				intel_pch = PCH_NONE;
 		} else {
 			printf("Dumping from file without -d argument. "
 			       "Assuming Ironlake machine.\n");
 			devid = 0x0042;
-			pch = PCH_IBX;
+			intel_pch = PCH_IBX;
 		}
 	} else {
 		pci_dev = intel_get_pci_device();
@@ -2956,7 +2959,7 @@ int main(int argc, char** argv)
 
 	power_well = power_well_get();
 
-	if (IS_HASWELL(devid)) {
+	if (IS_HASWELL(devid) || IS_BROADWELL(devid)) {
 		intel_dump_regs(haswell_debug_regs);
 	} else if (IS_GEN5(devid) || IS_GEN6(devid) || IS_IVYBRIDGE(devid)) {
 		intel_dump_regs(ironlake_debug_regs);
